pub mod assets;
pub mod config;
pub mod game;
mod menu;
pub mod particle;
pub mod sky;

use crate::assets::Assets;
use crate::config::{SCREEN_ASPECT, SCREEN_HEIGHT, SCREEN_WIDTH};
use crate::game::{GameState, GameWorld};
use crate::menu::GameMenu;
use macroquad::audio::AudioContext;
use macroquad::prelude::*;

fn window_conf() -> Conf {
    Conf {
        window_title: "Blobcat Delivery".parse().unwrap(),
        window_width: 1280,
        window_height: 720,
        ..Default::default()
    }
}

#[macroquad::main(window_conf)]
async fn main() {
    let _audio = AudioContext::new();
    let assets = Assets::load().await;

    let mut game_state = GameState::Menu(GameMenu::default());

    loop {
        let sw = screen_width();
        let sh = screen_height();
        let rr = sw / sh;
        let rw = if rr > SCREEN_ASPECT {
            SCREEN_ASPECT / rr
        } else {
            1.0
        };
        let rh = if rr > SCREEN_ASPECT {
            1.0
        } else {
            rr / SCREEN_ASPECT
        };

        set_camera(&Camera2D {
            zoom: vec2(
                1.0 / SCREEN_WIDTH * 2.0 * rw,
                1.0 / -SCREEN_HEIGHT * 2.0 * rh,
            ),
            target: vec2(SCREEN_WIDTH / 2.0, SCREEN_HEIGHT / 2.0),
            ..Default::default()
        });

        match &mut game_state {
            GameState::Game(game) => {
                clear_background(SKYBLUE);
                game.frame(&assets).await;
            }
            GameState::Menu(menu) => {
                clear_background(DARKBLUE);
                if menu.frame(&assets).await {
                    game_state = GameState::Game(GameWorld::default());
                }
            }
        }

        next_frame().await
    }
}
